unit EnvironmentUnit;

interface

uses
  Windows, Messages, SysUtils, Classes,
  Graphics, Controls, Forms, Dialogs,
  ColorGrd, StdCtrls, ExtCtrls, Buttons,
  ComCtrls, Spin, Grids,
  CE_Unit, ReuUnit, Menus;

const DIVIDER = ' | ';

type
  TEnvironmentForm = class(TForm)
    OKBtn: TBitBtn;
    CancelBtn: TBitBtn;
    Bevel1: TBevel;
    KeystrokeRG: TRadioGroup;
    StartWithRG: TRadioGroup;
    ColorRG: TRadioGroup;
    BackupExtRG: TRadioGroup;
    OpenSavePopup: TPopupMenu;
    InsertMI: TMenuItem;
    DeleteMI: TMenuItem;
    GroupBox1: TGroupBox;
    FiltersSG: TStringGrid;
    RestoreBtn: TButton;
    GroupBox2: TGroupBox;
    DefaultExtEdit: TEdit;
    GroupBox3: TGroupBox;
    ColorGrid1: TColorGrid;
    GroupBox4: TGroupBox;
    AutosaveCB: TCheckBox;
    IntervalLab: TLabel;
    IntervalSE: TSpinEdit;
    OrderBtn: TBitBtn;
    InsertBtn: TBitBtn;
    DeleteBtn: TBitBtn;
    PersistentGroupsCB: TCheckBox;
    ConfirmPromptCB: TCheckBox;
    HelpBtn: TBitBtn;
    procedure OKBtnClick(Sender: TObject);
    procedure ColorGrid1Change(Sender: TObject);
    procedure AutosaveCBClick(Sender: TObject);
    procedure FormShow(Sender: TObject);
    procedure IntervalSEKeyPress(Sender: TObject; var Key: Char);
    procedure RestoreBtnClick(Sender: TObject);
    procedure FormCreate(Sender: TObject);
    procedure InsertMIClick(Sender: TObject);
    procedure FiltersSGMouseDown(Sender: TObject; Button: TMouseButton;
      Shift: TShiftState; X, Y: Integer);
    procedure DeleteMIClick(Sender: TObject);
    procedure InsertBtnClick(Sender: TObject);
    procedure DeleteBtnClick(Sender: TObject);
    procedure DelBtnClick(Sender: TObject);
    procedure FiltersSGEnter(Sender: TObject);
    procedure FiltersSGExit(Sender: TObject);
    procedure OrderBtnClick(Sender: TObject);
    procedure FormKeyDown(Sender: TObject; var Key: Word;
      Shift: TShiftState);
  private
    { Private declarations }
    procedure FilterToGrid (Filter : string);
    function GridToFilter : string;
    procedure SetUp;
    procedure ChangeAuto;
    function RowsToString (aRow : integer) : string;
    function StringToRows (aRow : string) : TStringList;
  public
    { Public declarations }
  end;

var
  EnvironmentForm: TEnvironmentForm;

implementation

uses MainUnit, OrderUnit;

{$R *.DFM}

procedure TEnvironmentForm.ChangeAuto;
begin
  IntervalSE.Enabled := AutoSaveCB.Checked;
  IntervalLab.Enabled := AutoSaveCB.Checked;
end;

function TEnvironmentForm.RowsToString (aRow : integer) : string;
var i : integer;
begin
  with FiltersSG do begin
    if (aRow < 0) or (aRow >= RowCount) then exit;

    result := Cells[0, aRow];
    for i := 1 to ColCount - 1 do
      if Cells[i, aRow] = '' then
        break
      else
        result := result + DIVIDER + Cells[i, aRow];

  end;//with...
end;

function TEnvironmentForm.StringToRows (aRow : string) : TStringList;
var i, DivLen : cardinal;
begin
   result := TStringList.Create;
   i := pos (DIVIDER, aRow);
   if i = 0 then begin
     result.Add (aRow);
     exit;
   end;

   result.Add (copy (aRow, 1, i - 1));
   DivLen := length (DIVIDER);
   result.Add (copy (aRow, i + DivLen, length (aRow) - i - DivLen + 1));
end;

procedure TEnvironmentForm.FilterToGrid (Filter : string);
var start, stop, len, aRow, aCol : cardinal;
begin
  with FiltersSG do begin
    Cells[0, 0] := 'Filter Name';
    Cells[1, 0] := 'Filter';
    len := length (Filter);
    if len = 0 then exit;
    start := 1;
    aRow := 1;
    aCol := 0;
    stop := 1;
    while stop < len do begin
      stop := posL ('|', Filter, start, len);
      if stop = 0 then stop := len + 1;

      Cells [aCol, aRow] := copy (Filter, start, stop - start);

      start := stop + 1;
      inc (aCol);
      if aCol = ColCount then begin
        aCol := 0;
        inc (aRow);
        if aRow > RowCount then exit;
      end;
    end; // while...

    {Set the rest of FiltersSG's strings to
     ''}
    while aRow < RowCount do begin
      Cells[aCol, aRow] := '';
      inc (aCol);
      if aCol = ColCount then begin
        aCol := 0;
        inc (aRow);
      end; // if...
    end; // while...

  end; // with StringGrid...
end;

function TEnvironmentForm.GridToFilter : string;
var aRow, aCol : cardinal;
begin
  {Copy each string of FiltersSG to Filter
   starting at the second row (so column
   titles aren't copied) and putting a '|'
   after each string}
  with FiltersSG do
    for aRow := 1 to RowCount - 1 do
      //don't insert a '|' in a blank row
      if Rows[aRow].Text <> '' then
        for aCol := 0 to ColCount - 1 do
          result := result + Cells[aCol, aRow] + '|';

  //delete probible trailing '|'
  aCol := length (result);
  if result[aCol] = '|' then delete (result, aCol, 1);
end;

procedure TEnvironmentForm.SetUp;
begin
  with MainForm do begin
    ColorGrid1.ForeGroundIndex := ColorToIndex (CustomFore);
    ColorGrid1.BackGroundIndex := ColorToIndex (CustomBack);

    ColorRG.ItemIndex := Ord (ColorScheme);

    KeystrokeRG.ItemIndex := Ord (KeyMap);
    AutoSaveCB.Checked := Timer1.Enabled;
    ChangeAuto;
    IntervalSE.Value := Round (Timer1.Interval div 60000);
    StartWithRG.ItemIndex := StartWith;

    BackupExtRG.ItemIndex := Ord (BackupExt);

    DefaultExtEdit.Text := SaveDialog1.DefaultExt;
    FilterToGrid (SaveDialog1.Filter);

    PersistentGroupsCB.Checked := PersistentGroups;
    ConfirmPromptCB.Checked := ConfirmPrompt;
  end;
end;

procedure TEnvironmentForm.OKBtnClick(Sender: TObject);
begin
  with MainForm do begin
    if KeystrokeRG.ItemIndex = 0 then
      ChangeKeyMap (kmWindows)
    else
      ChangeKeyMap (kmPascal);

    CustomBack := ColorGrid1.BackGroundColor;
    CustomFore := ColorGrid1.ForeGroundColor;
    AssignColorScheme (ColorRG.ItemIndex);

    Timer1.Interval := IntervalSE.Value * 60000;
    Timer1.Enabled := AutoSaveCB.Checked;

    StartWith := StartWithRG.ItemIndex;

    BackupExt := IntToBackupExt (BackupExtRG.ItemIndex);

    {Erase beginning . if there is one}
    if DefaultExtEdit.GetTextLen > 0 then
      if DefaultExtEdit.Text[1] = '.' then
        DefaultExtEdit.Text := copy (DefaultExtEdit.text, 2, DefaultExtEdit.GetTextLen - 1);

    {the following code is remarked out since I think
     extensions more than 3 letters long usually work}
    {Prompts user to change default extension
     if it's > 3 chars long}
    {if (DefaultExtEdit.GetTextLen > 3)
    and not (StrCmp (DefaultExtEdit.Text, SaveDialog1.DefaultExt)) then
      if MessageDlg ('If the default extension is more then three characters long, it may not (but then again it might) work properly.  Do you want to change it?',
         mtWarning, [mbYes, mbNo], 0) = mrNo then begin
        DefaultExtEdit.SetFocus;
        DefaultExtEdit.SelectAll;
        EnvironmentForm.ModalResult := mrNone;
        exit;
      end;}

    SaveDialog1.DefaultExt := DefaultExtEdit.Text;
    OpenDialog1.DefaultExt := DefaultExtEdit.Text;
    SaveDialog1.Filter := GridToFilter;
    OpenDialog1.Filter := SaveDialog1.Filter;

    PersistentGroups := PersistentGroupsCB.Checked;
    ConfirmPrompt := ConfirmPromptCB.Checked;
  end;
end;

procedure TEnvironmentForm.ColorGrid1Change(Sender: TObject);
begin
  ColorRG.ItemIndex := 2; //Custom
end;

procedure TEnvironmentForm.AutoSaveCBClick(Sender: TObject);
begin
  ChangeAuto;
end;

procedure TEnvironmentForm.FormShow(Sender: TObject);
begin
  SetUp;
end;

procedure TEnvironmentForm.IntervalSEKeyPress(Sender: TObject;
  var Key: Char);
begin
  ValidateKey (Key);
end;

procedure TEnvironmentForm.RestoreBtnClick(Sender: TObject);
begin
  if MessageDlg ('Restore filters to their default setting?', mtConfirmation, [mbYes, mbNo], 0) = mrNo then exit;
  FilterToGrid (DefaultFilters);
end;

procedure TEnvironmentForm.FormCreate(Sender: TObject);
begin
  // AutoSize FiltersSG
  FiltersSG.DefaultRowHeight := DefaultExtEdit.Height;
end;

//inserts a blank row in FiltersSG
procedure TEnvironmentForm.InsertMIClick(Sender: TObject);
begin
  InsertBtn.Click;
end;

procedure TEnvironmentForm.FiltersSGMouseDown(Sender: TObject;
  Button: TMouseButton; Shift: TShiftState; X, Y: Integer);
var aCol, aRow : LongInt;
begin
  {when the user right click a popup menu appears; this
   ensures the cell that was right clicked on is the
   selected cell}
  if Button = mbRight then begin
    FiltersSG.MouseToCell (X, Y, aCol, aRow);
    if aRow > 0 then begin //don't select Col headers
      FiltersSG.Col := aCol;
      FiltersSG.Row := aRow;
    end;
  end;
end;

procedure TEnvironmentForm.DeleteMIClick(Sender: TObject);
begin
  DeleteBtn.Click;
end;

procedure TEnvironmentForm.InsertBtnClick(Sender: TObject);
var aRow, aCol : cardinal;
begin
  with FiltersSG do begin
    //confirm is insertion will delete last row
    if Cells[0, RowCount - 1] <> '' then
      if MessageDlg ('Inserting a new row will delete the last row. Do you want to insert a new row anyway?', mtConfirmation, [mbYes, mbNo], 0) = mrNo then
        exit;

    //move rows up to selected row down 1 row
    {- 2 because I want to start at second to last row and
     because Cells is 0 based}
    for aRow := RowCount - 2 downto Row do
      for aCol := 0 to ColCount - 1 do
        Cells[aCol, aRow + 1] := Cells[aCol, aRow];

    //clear out selected row
    for aCol := 0 to ColCount - 1 do
      Cells[aCol, Row] := '';
  end;//with...
end;

procedure TEnvironmentForm.DeleteBtnClick(Sender: TObject);
var aRow, aCol : cardinal;
begin
  with FiltersSG do begin
    //move rows up to selected row up 1 row
    {- 2 because I want to end at second to last row and
     because Cells is 0 based}
    for aRow := Row to RowCount - 2 do
      for aCol := 0 to ColCount - 1 do
        Cells[aCol, aRow] := Cells[aCol, aRow + 1];

    //clear out last row
    for aCol := 0 to ColCount - 1 do
      Cells[aCol, RowCount - 1] := '';
  end;//with...
end;

procedure TEnvironmentForm.DelBtnClick(Sender: TObject);
var aRow, aCol : cardinal;
begin
  with FiltersSG do begin
    //move rows up to selected row up 1 row
    {- 2 because I want to end at second to last row and
     because Cells is 0 based}
    for aRow := Row to RowCount - 2 do
      for aCol := 0 to ColCount - 1 do
        Cells[aCol, aRow] := Cells[aCol, aRow + 1];

    //clear out last row
    for aCol := 0 to ColCount - 1 do
      Cells[aCol, RowCount - 1] := '';
  end;//with...
end;

procedure TEnvironmentForm.FiltersSGEnter(Sender: TObject);
begin
  FiltersSG.Options := FiltersSG.Options + [goAlwaysShowEditor];
end;

procedure TEnvironmentForm.FiltersSGExit(Sender: TObject);
begin
  FiltersSG.Options := FiltersSG.Options - [goAlwaysShowEditor];
end;

procedure TEnvironmentForm.OrderBtnClick(Sender: TObject);
var
  i : cardinal;
  s : string;
begin
  OrderForm.MainItem := -1;

  OrderForm.ListBox1.Clear;
  i := 1;
  while (i < FiltersSG.RowCount) do begin
     s := RowsToString (i);
     if s = '' then break;
     OrderForm.ListBox1.Items.Add (s);
     inc (i);
  end;

  if OrderForm.ShowModal = mrCancel then exit;

  for i := 1 to OrderForm.ListBox1.Items.Count - 1 do
    FiltersSG.Rows[i] := StringToRows (OrderForm.ListBox1.Items[i - 1]);
end;



procedure TEnvironmentForm.FormKeyDown(Sender: TObject; var Key: Word;
  Shift: TShiftState);
begin
  if Key = VK_F1 then HelpBtn.Click;
end;

end.

